/*
    Copyright (c) 2010-2011 250bpm s.r.o.
    Copyright (c) 2010-2011 Other contributors as noted in the AUTHORS file

    This file is part of 0MQ.

    0MQ is free software; you can redistribute it and/or modify it under
    the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.

    0MQ is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

using System;
using System.Net.Sockets;
using System.Threading;
using System.Security.AccessControl;
using System.Diagnostics;
using System.Net;
using System.Runtime.InteropServices;

//  This is a cross-platform equivalent to signal_fd. However, as opposed
//  to signal_fd there can be at most one signal in the signaler at any
//  given moment. Attempt to send a signal before receiving the previous
//  one will result in undefined behaviour.

namespace NetMQ.zmq
{
    public class Signaler
    {
        //  Underlying write & read file descriptor.
        private Socket m_w;
        private Socket m_r;
        private byte[] m_dummy;

        public Signaler()
        {
            m_dummy = new byte[1]{0};

            //  Create the socketpair for signaling.
            MakeFDpair();

            //  Set both fds to non-blocking mode.
            m_w.Blocking = false;
            m_r.Blocking = false;
        }

        public void Close()
        {
            try
            {
                m_w.LingerState = new LingerOption(true, 0);
            }
            catch (SocketException)
            {                                
            }

            try
            {
                
                m_w.Close();
            }
            catch (SocketException)
            {
            }

            try
            {
                m_r.Close();
            }
            catch (SocketException)
            {
            }
        }

        //  Creates a pair of filedescriptors that will be used
        //  to pass the signals.
        private void MakeFDpair()
        {            
            Socket listner = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Unspecified);
            listner.NoDelay = true;
            listner.SetSocketOption(SocketOptionLevel.Socket, SocketOptionName.ReuseAddress, true);

            // using ephemeral port            
            listner.Bind(new IPEndPoint(IPAddress.Loopback, 0));
            listner.Listen(1);
            
            m_w = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Unspecified);

            m_w.NoDelay = true;

            m_w.Connect(listner.LocalEndPoint);

            m_r = listner.Accept();

            listner.Close();          
        }

        public Socket FD
        {
            get
            {
                return m_r;
            }
        }

        public void Send()
        {            
            int nbytes = m_w.Send(m_dummy);

            Debug.Assert(nbytes == 1);
        }

        public bool WaitEvent(int timeout)
        {
            if(m_r.Connected)
                return m_r.Poll(timeout * 1000, SelectMode.SelectRead);
            return false;
        }

        public void Recv()
        {
            byte[] dummy = new byte[1];

            int nbytes = m_r.Receive(dummy);

            Debug.Assert(nbytes == 1);
            Debug.Assert(dummy[0] == 0);
        }




    }
}
